//=======================================================================//
// OPENCDLG.CPP released to the public domain 6/6/92 by Bob Bourbonnais  //
//     Demonstrates using a Windows 3.1 File Open Common Dialog Box      //
//             with C++ and the Object Windows Class Library             //
//   For the best demonstration use Opencdlg.tdl as the input file.      //
//=======================================================================//
#include <owl.h>        // for Object Windows
#include <listbox.h>    // for listbox
#include <dialog.h>     // for dialog boxes
#include <commdlg.h>    // for Win 3.1 Common Dialogs
#include "opencdlg.h"
//=======================================================================//
void lpstrcat(LPSTR DestString, int *DestLength,        // Rewrite of
	      LPSTR SourceString, int SourceLength)     // string cat to
  {                                                     // allow null
  int Count;                                            // characters to
  for (Count=0;Count<SourceLength;Count++)              // be embedded within
    {                                                   // the string to
    DestString[*DestLength+Count] = SourceString[Count];// delimit fields
    }
  *DestLength += Count;
  };
//=======================================================================//
_CLASSDEF(TOpenDialog)               // Class to encapsulate
class TOpenDialog : public TDialog   // the Windows 3.1
  {                                  // Font Common Dialog
  public:                            // needs lpstrcat
    TOpenDialog(PTWindowsObject AParent, LPSTR AName,  // Constructor
		  char far * szFileName, char (*aszFilterArray)[40],
		  PTModule AModule);
    virtual BOOL Create();            // Create for non-modal Creation
    virtual int Execute();            // Execute for modal Execution
    #pragma argsused                  // Ignore ARetValue non use
    virtual void CloseWindow(int ARetValue){}; // trap destruction
  public:
    OPENFILENAME ocdlg;                // Open File Common Dialog Structure
    char szFile[256];                  // File Name variable
    char szFilter[256];                // File Types
  };
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
TOpenDialog::TOpenDialog(PTWindowsObject AParent, LPSTR AName,
			 char far * szFileName,      // In=Search,Out=FileName
			 char (*aszFilterArray)[40], // File Types
			 PTModule AModule = NULL)
	     :TDialog(AParent,AName,AModule) // Constructor calls base
  {
  int nFilterLength=0;                          // pack the filter array
  int Count, nLimit = 0;                        // into null field delimited
  while (lstrcmp(aszFilterArray[nLimit],"")!=0) // string that file cdlg
    nLimit++;                                   // needs using lpstrcat
  for (Count=0;Count<=nLimit;Count++)           // function defined earlier
    lpstrcat(szFilter,&nFilterLength,           // in this program
	     aszFilterArray[Count],
	     lstrlen(aszFilterArray[Count])+1);

  ocdlg.lStructSize       = sizeof(OPENFILENAME);  // initialize ocdl struct
  ocdlg.hwndOwner         = GetFocus();            // parent has the focus
  ocdlg.lpstrFilter       = (LPSTR)szFilter;       // Filter on startup
  ocdlg.lpstrCustomFilter = (LPSTR)NULL;           // No Filter history
  ocdlg.nMaxCustFilter    = 0L;                    // No Filter history
  ocdlg.nFilterIndex      = 1L;                    // Use first filter
  ocdlg.lpstrFile         = (LPSTR)szFileName;     // File name I/O w path
  ocdlg.nMaxFile          = 255;                   // Max file name size
  ocdlg.lpstrFileTitle    = (LPSTR)NULL;           // No Name only output
  ocdlg.nMaxFileTitle     = 0L;                    // No Name only size
  ocdlg.lpstrInitialDir   = (LPSTR)NULL;           // Current Directory
  ocdlg.lpstrTitle        = (LPSTR)NULL;           // Title of Dialog Box
  ocdlg.Flags  = OFN_FILEMUSTEXIST;                // Valid File name only
  ocdlg.nFileOffset       = 0;                     // Path offset
  ocdlg.nFileExtension    = 0;                     // Filename offset
  ocdlg.lpstrDefExt       = (LPSTR)NULL;           // Default file extension
  };
BOOL TOpenDialog::Create()                         // Called by MakeWindow
  {
  ocdlg.hwndOwner = NULL;                          // No parent for non-Model
  GetOpenFileName(&ocdlg);                         // fcdlg function call
  return 1;                                        // Success
  }
int TOpenDialog::Execute()                         // Called by CreateDialog
  {
  GetOpenFileName(&ocdlg);                         // fcdlg function call
  return 1;                                        // Success
  }
//=======================================================================//
_CLASSDEF(TMainDialog)                   // Class derived from a dialog
class TMainDialog : public TDialog       // to add menu and button
  {					 // processing to a main dialog
  public:
    TListBox * pListBox;                 // Pointer for List Box
    TMainDialog(LPSTR lpName);	         // constructor
    virtual void HandleMenuItem(RTMessage Msg)      // menu handler
      = [CM_FIRST + IDM_ADD_DATA];
    virtual void HandleButtonMessage(RTMessage Msg) // button handler
      = [ID_FIRST + IDB_ADD_DATA];
  protected:
    char szFileIO[255];
    OFSTRUCT FileStruct;
  };
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
TMainDialog::TMainDialog(LPSTR lpName)              // constructor
      :TDialog(NULL,lpName) 		            // calls base class
  {
  lstrcpy(szFileIO,"*.tdl");
  }
void TMainDialog:: HandleMenuItem(RTMessage)          // menu item activates
  {                                                   // non-modal common
  MessageBox(GetFocus(),                              // open dialog
	     "Use Opencdlg.tdl found in the same directory as opencdlg.exe",
	     "For best test results", MB_OK);
  char aszFileTypes[][40] =                           // file types passed
    {                                                 // to file open
    "Tab Delimited (*.tdl)","*.tdl",                  // common dialog
    "Text Files (*.txt)"   ,"*.txt",
    "Data Files (*.dat)"   ,"*.dat",
    "All Files (*.*)"      ,"*.*",                    // last array element
    ""                                                // must be ""
    };
  GetApplication()->MakeWindow(new TOpenDialog(this,"Main_Window_Dialog",
					       szFileIO,aszFileTypes));
  char szBigBuff[1024];                // Read file into buffer
  HFILE hFile;
  hFile = OpenFile(szFileIO,&FileStruct,OF_READ);
  if (hFile == -1)
    MessageBox(GetFocus(),szFileIO,"Open File Failed for",MB_OK);
  else
    {
    int nBigLimit = _lread(hFile,szBigBuff,1023);
    _lclose(hFile);
    char szSmallBuff[255];
    int nSmallCount = 0;
    int nBigCount = 0;
    int TabStops[] = {10,101,255};       // Set Tab Stops in list box
    SendDlgItemMsg(ID_LISTBOX,LB_SETTABSTOPS,3,(LONG)(LPSTR)TabStops);
    while (nBigCount < nBigLimit)        // read buffer into list box
      {
      while ((szBigBuff[nBigCount] != '\n')&(nSmallCount<255))
	szSmallBuff[nSmallCount++] = szBigBuff[nBigCount++];
      szSmallBuff[nSmallCount-1] = NULL; // Add Null Terminator
      SendDlgItemMsg(ID_LISTBOX,LB_ADDSTRING,NULL,(LONG)szSmallBuff);
      nSmallCount = 0;
      nBigCount++;
      }
    }
  }
void TMainDialog:: HandleButtonMessage(RTMessage)     // button activates
  { 						      // modal common open
  MessageBox(GetFocus(),                              // dialog
	     "Use Opencdlg.tdl found in the same directory as opencdlg.exe",
	     "For best test results", MB_OK);
  char aszFileTypes[][40] =                           // file types passed
    {                                                 // to file open
    "Tab Delimited (*.tdl)","*.tdl",                  // common dialog
    "Text Files (*.txt)"   ,"*.txt",
    "Data Files (*.dat)"   ,"*.dat",
    "All Files (*.*)"      ,"*.*",                    // last array element
    ""                                                // must be ""
    };
  GetApplication()->ExecDialog(new TOpenDialog(this,"Main_Window_Dialog",
					       szFileIO,aszFileTypes));
  char szBigBuff[1024];                // Read file into buffer
  HFILE hFile;
  hFile = OpenFile(szFileIO,&FileStruct,OF_READ);
  if (hFile == -1)
    MessageBox(GetFocus(),szFileIO,"Open File Failed for",MB_OK);
  else
    {
    int nBigLimit = _lread(hFile,szBigBuff,1023);
    _lclose(hFile);
    char szSmallBuff[255];
    int nSmallCount = 0;
    int nBigCount = 0;
    int TabStops[] = {10,101,255};       // Set Tab Stops in list box
    SendDlgItemMsg(ID_LISTBOX,LB_SETTABSTOPS,3,(LONG)(LPSTR)TabStops);
    while (nBigCount < nBigLimit)        // read buffer into list box
      {
      while ((szBigBuff[nBigCount] != '\n')&(nSmallCount<255))
	szSmallBuff[nSmallCount++] = szBigBuff[nBigCount++];
      szSmallBuff[nSmallCount-1] = NULL; // Add Null Terminator
      SendDlgItemMsg(ID_LISTBOX,LB_ADDSTRING,NULL,(LONG)szSmallBuff);
      nSmallCount = 0;
      nBigCount++;
      }
    }
  }
//=======================================================================//
class TStandardApp : public TApplication  // Application Class to contain
  {                                       // the application
  public:
    TStandardApp(LPSTR lpName, HINSTANCE hInstance, // constructor calls the
		HINSTANCE hPrevInstance,            // base class constructor
		LPSTR lpCmdLine, int nCmdShow)
		:TApplication(lpName, hInstance,
			      hPrevInstance,
			      lpCmdLine, nCmdShow) {};

    virtual void InitMainWindow(); // overrides base class InitMainWindow
  };
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
void TStandardApp::InitMainWindow() // to initialize a dialog box
  {                                 // as the main window
  MainWindow = new TMainDialog("Main_Window_Dialog");
  }
/*************************************************************************/
int PASCAL WinMain(HINSTANCE hInstance,              // main entry point from
		   HINSTANCE hPrevInstance,          // windows to this program
		   LPSTR lpCmdLine , int nCmdShow)
  {
  TStandardApp AppDialog("Dialog Tester",hInstance, // create instance of
		       hPrevInstance,               // the dialog application
		       lpCmdLine,nCmdShow);
  AppDialog.Run();                                  // run it
  return (AppDialog.Status);                        // exit
  }
/*************************************************************************/
//   Remember to use IMPLIB on COMDLG.DLL to create an import library    //
//       And then include the COMMDLG.LIB in the Project Window          //
/*************************************************************************/